<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\AdminSalesInvoice;
use App\AdminPurchaesInvoice;
use App\CreditNote;
use App\DebitNote;
use App\OpeningBalance;
use App\AdminSalesInvoiceExpense;
use App\AdminTermsAndCondition;
use App\AdminWarehouse;
use App\Company;
use App\Customer;
use App\DeliveryNote;
use App\Expense;
use App\ItemName;
use App\OtherAccountBuyer;
use App\Quotation;
use App\StaffAndSalesman;
use App\Supplier;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Permission;
use App\Branches;
use App\AdminPeriodClosing;
use App\User;
use App\AccountMasterPermission;
use Auth;

class AdminSalesInvoiceController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {

        $userId = Auth::id();

        $branchId = session('selected_branch');

        $permissionIds = AccountMasterPermission::where([
            ['user_id', $userId],
            ['branch_id', $branchId],
        ])->pluck('account_master_no');

        // whereIn('acc_no', $permissionIds)

        $salesInvoice = AdminSalesInvoice::with('account', 'item')
        ->whereIn('acc_no', $permissionIds)->whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                ->from('admin_sales_invoices')
                ->where('branch_id', session()->get('selected_branch'))
                ->whereNotNull('invoice_no')
                ->groupBy('invoice_no');
        })
        ->withTrashed()
        ->orderBy('invoice_no', 'desc')->get();

        return view('crusher.trans_sale_invoice.index', compact('salesInvoice'));
    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {



         // Retrieve permission titles from Permission table
         $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

         // Filter permission titles using hasBranchAccountPermission function
         $filteredPermissionTitles = [];
         $branchId = session()->get('selected_branch');
         foreach ($permissionTitles as $permissionTitle) {
             if (Branches::hasBranchAccountPermission($branchId, $permissionTitle)) {
                 $filteredPermissionTitles[] = $permissionTitle;
             }
         }

        $deliveryNotes = collect();
        $quotations = collect();

        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'6'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'6'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        $currentDate = Carbon::now()->format('Ymd');
        // Get the latest quotation number for the branch
        $latestQuotation = Quotation::withTrashed()->where('branch_id', session()->get('selected_branch'))
            ->latest('quotation_no')->first();

        // Calculate the new quotation number
        if ($latestQuotation) {
            $lastQuotationNumber = explode('-', $latestQuotation->quotation_no)[1];
            $newQuotationNumber = $currentDate . '-' . str_pad((int)$lastQuotationNumber + 1, 3, '0', STR_PAD_LEFT);
        } else {
            $newQuotationNumber = $currentDate . '-001';
        }

        // Get the latest quotation number for the branch
        $latestDo = DeliveryNote::withTrashed()->where('branch_id', session()->get('selected_branch'))
        ->latest('do_no')->first();

        // Calculate the new quotation number
        if ($latestDo) {
            $lastDoNumber = explode('-', $latestDo->do_no)[1];
            $newDoNumber = 'DO#'.$currentDate . '-' . str_pad((int)$lastDoNumber + 1, 3, '0', STR_PAD_LEFT);
        } else {
            $newDoNumber = 'DO#'.$currentDate . '-001';
        }

         // Get the latest quotation number for the branch
         $latestSi = AdminSalesInvoice::withTrashed()->where('branch_id', session()->get('selected_branch'))
         ->latest('invoice_no')->first();

         // Calculate the new quotation number
         if ($latestSi) {
             $lastNumber = (int)substr($latestSi->invoice_no, -4);
             $newNumber = $lastNumber + 1;
             $newSiNumber = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
         } else {
             $newSiNumber = '0001';
         }

        $doNo = request()->input('do_no');

        if ($doNo) {
            $deliveryNotes = DeliveryNote::where('do_no',$doNo)->get();
            $quotations = Quotation::with('expenses')->where('quotation_no',$deliveryNotes->first()->quotation_no)->get();
        }

        $quotation = request()->input('quotation_no');

        if ($quotation) {
            $quotations = Quotation::with('expenses')->where('quotation_no',$quotation)->get();
        }

        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'sales_invoice'])->first();



        //  // Retrieve customers and include both 'account_name' and 'table_name' attributes
        //  $customers = Customer::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        //  ->select('*', 'name_of_organisation as account_name')
        //  ->get()
        //  ->map(function ($item) {
        //      return $item->setAttribute('table_name', 'App\Customer');
        //  });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        // ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        // ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        // });

        //  // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        //  $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        //  ->select('*', 'name_of_organisation as account_name')
        //  ->get()
        //  ->map(function ($item) {
        //      return $item->setAttribute('table_name', 'App\Supplier');
        //  });

        //  // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        //  $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        //  ->get()
        //  ->map(function ($item) {
        //      return $item->setAttribute('table_name', 'App\Expense');
        //  });

        //  // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        //  $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        //  ->get()
        //  ->map(function ($item) {
        //      return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        //  });

        $userId = Auth::id();

        $branchId = session('selected_branch');

        $permissionIds = AccountMasterPermission::where([
            ['user_id', $userId],
            ['branch_id', $branchId],
        ])->pluck('account_master_no');

        // whereIn('acc_no', $permissionIds)

        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) AS account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            $item->setAttribute('table_name', 'App\AdminOtherAccount');
            $item->setAttribute('class_of_account', ($item->class_of_account_id == 3) ? '3' : '');
            return $item;
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        //  dd($accounts);

        $periodClosings = AdminPeriodClosing::where('branch_id', session()->get('selected_branch'))->pluck('to')->toArray();

        // $periodClosings = AdminPeriodClosing::where('branch_id', session()->get('selected_branch'))->get('to');

        // dd($periodClosings);

        return view('crusher.trans_sale_invoice.create', compact('periodClosings','tnc', 'customers', 'items', 'quotations', 'newSiNumber', 'warehouses', 'expensesAccount', 'deliveryNotes', 'accounts'));
    }

    public function checkStock(Request $request) {

        $item_stock = $request->itemId;
        $quantity = $request->quantity;

        $openingBalanceForInventories = OpeningBalance::where('branch_id', session()->get('selected_branch'))
            ->where('item_id', $item_stock)
            ->groupBy('item_id')
            ->whereNotNull('invoice_no')
            ->sum('quantity');

        $purchaseInvoices = AdminPurchaesInvoice::where('branch_id', session()->get('selected_branch'))
            ->where('item_id', $item_stock)
            ->groupBy('item_id')
            ->whereNotNull('invoice_no')
            ->sum('quantity');

        $salesInvoices = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
            ->where('item_id', $item_stock)
            ->groupBy('item_id')
            ->whereNotNull('invoice_no')
            ->sum('quantity');

        $purchaseReturnInvoices = DebitNote::where('branch_id', session()->get('selected_branch'))
            ->where('item_id', $item_stock)
            ->groupBy('item_id')
            ->whereNotNull('invoice_no')
            ->sum('quantity');

        $salesReturnInvoices = CreditNote::where('branch_id', session()->get('selected_branch'))
            ->where('item_id', $item_stock)
            ->groupBy('item_id')
            ->whereNotNull('invoice_no')
            ->sum('quantity');

        $pp = ($openingBalanceForInventories + $purchaseInvoices) + $salesReturnInvoices;

        $ss = ($salesInvoices + $salesReturnInvoices) + $purchaseReturnInvoices;

        $remainingQty = $pp - $ss;

        // Check if remaining quantity is less than the entered quantity
        $outOfStock = ($remainingQty < $quantity);

        // dd($remainingQty);

        return response()->json(['outOfStock' => $outOfStock]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */


     public function store(Request $request)
     {
        // dd($request);
        $latestNo = AdminSalesInvoice::withTrashed()->where('branch_id', session()->get('selected_branch'))
        ->latest('invoice_no')->first();

        if ($latestNo) {
        $latestInvoiceNo = $latestNo->invoice_no;

        $latestNumber = (int)preg_replace('/[^0-9]/', '', $latestInvoiceNo);

        $newNumber = $latestNumber + 1;

        $newInvoiceNo = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
        $newInvoiceNo = '0001';
        }
         try {
             $validatedData = $request->validate([
                //  'invoice_no' => 'nullable|string',
                 'quotation_no' => 'nullable|string',
                 'do_no' => 'nullable|string',
                 'lpo_no' => 'nullable|string',
                 'lpo_days' => 'nullable|string',
                 'term_of_payment' => 'nullable|string',
                 'narrations' => 'nullable|string',
                 'date' => 'nullable|date',
                 'account_id' => 'nullable',
                 'acc_no' => 'nullable',
                 'account_type' => 'nullable',
                 'name_account' => 'nullable',
                 'item_description' => 'nullable|array',
                 'item_description.*' => 'nullable|string',
                 'item_id' => 'nullable|array',
                 'item_id.*' => 'nullable|exists:item_names,id',
                 'item_unit' => 'nullable|array',
                 'item_unit.*' => 'nullable|string',
                 'warehouse_id' => 'nullable|array',
                 'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                 'quantity' => 'nullable|array',
                 'quantity.*' => 'nullable|numeric',
                 'rate_unit' => 'nullable|array',
                 'rate_unit.*' => 'nullable|numeric',
                 'gross_amount' => 'nullable|array',
                 'gross_amount.*' => 'nullable|numeric',
                 'vat_percentage' => 'nullable|array',
                 'vat_percentage.*' => 'nullable|numeric',
                 'vat_amount' => 'nullable|array',
                 'vat_amount.*' => 'nullable|numeric',
                 'net_amount' => 'nullable|array',
                 'net_amount.*' => 'nullable|numeric',
                 'expense_type' => 'nullable|array',
                 'expense_type.*' => 'nullable',
                 'account_name' => 'nullable|array',
                 'account_name.*' => 'nullable',
                 'expense_id' => 'nullable|array',
                 'expense_id.*' => 'nullable',
                 'expense_amount' => 'nullable|array',
                 'expense_amount.*' => 'nullable|numeric',
                 'total_quantity' => 'nullable|numeric',
                 'total_gross_amount' => 'nullable|numeric',
                 'total_vat_amount' => 'nullable|numeric',
                 'total_net_amount' => 'nullable|numeric',
                 'AmountInWords' => 'nullable|string',
                 'discount' => 'nullable',
             ]);

             $branch_id = session()->get('selected_branch');


             // Iterate over the item descriptions
             for ($i = 0; $i < count($validatedData['quantity']); $i++) {


                $invoiceData = [
                    'invoice_no' => $newInvoiceNo,
                    'quotation_no' => $validatedData['quotation_no'],
                    'do_no' => $validatedData['do_no'],
                    'lpo_no' => $validatedData['lpo_no'],
                    'lpo_days' => $validatedData['lpo_days'],
                    'term_of_payment' => $validatedData['term_of_payment'],
                    'narrations' => $validatedData['narrations'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'account_name' => $validatedData['name_account'],
                    'acc_no' => $validatedData['acc_no'],
                    'item_description' => $validatedData['item_description'][$i] ?? (NULL),
                    'quantity' => $validatedData['quantity'][$i] ?? (NULL),
                    'rate_unit' => $validatedData['rate_unit'][$i] ?? (NULL),
                    'item_unit' => $validatedData['item_unit'][$i] ?? (NULL),
                    'gross_amount' => $validatedData['gross_amount'][$i] ?? (NULL),
                    'vat_percentage' => $validatedData['vat_percentage'][$i] ?? (NULL),
                    'vat_amount' => $validatedData['vat_amount'][$i] ?? (NULL),
                    'net_amount' => $validatedData['net_amount'][$i] ?? (NULL),
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'AmountInWords' => $validatedData['AmountInWords'],
                    'discount' => $validatedData['discount'],
                    'branch_id' => $branch_id,
                    'item_id' => $validatedData['item_id'][$i] ?? (NULL),
                    'warehouse_id' => $validatedData['warehouse_id'][$i] ?? (NULL),
                ];

                $invoice = AdminSalesInvoice::create($invoiceData);


                if (isset($validatedData['expense_id']) && count($validatedData['expense_id']) > 0) {
                    for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                        AdminSalesInvoiceExpense::create([
                            'admin_sales_invoice_id' => $invoice->id,
                            'expense_type' => $validatedData['expense_type'][$j] ?? null,
                            'account_name' => $validatedData['account_name'][$j] ?? null,
                            'acc_no' => $validatedData['acc_no'][$j] ?? null,
                            'expense_id' => $validatedData['expense_id'][$j] ?? null,
                            'expense_amount' => $validatedData['expense_amount'][$j] ?? null,
                        ]);
                    }
                }

                if (isset($request['name']) && !empty($request['name'])) {
                    OtherAccountBuyer::firstOrCreate([
                        'invoice_no' => $newInvoiceNo,
                        'type' => $request->type,
                        'name' => $request->name,
                        'phone' => $request->phone,
                        'email' => $request->email,
                        'vat_number' => $request->vat_number,
                        'description' => $request->description,
                        'address' => $request->address,
                    ]);
                }


        }

             flash()->addSuccess('Record Created');
             return redirect()->route('software.admin_sales_invoice.index');
         } catch (\Throwable $th) {
             flash()->addError($th->getMessage());
             return redirect()->back()->withInput();
         }
     }

    // public function show($id)
    // {
    //     // Use withTrashed() to include soft-deleted records
    //     $adminSalesInvoice = AdminSalesInvoice::withTrashed()->findOrFail($id);

    //     dd($adminSalesInvoice);
    // }
    public function show($id)
    {

        $adminSalesInvoice = AdminSalesInvoice::withTrashed()->find($id);
        // dd($adminSalesInvoice);

        $salesInvoice = $adminSalesInvoice->withTrashed()->with('account', 'item', 'warehouse', 'expenses', 'buyer')->where('invoice_no', $adminSalesInvoice->invoice_no)->where('branch_id', session()->get('selected_branch'))->get();

        // $salesInvoice = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
        // ->with('account', 'item', 'warehouse', 'expenses', 'buyer')
        // ->where('invoice_no', $adminSalesInvoice->invoice_no)
        // ->get();


        // return $salesInvoice;
        // $company = Company::where('user_id', auth()->id())->first();

        $company_id = Branches::where('id', session()->get('selected_branch'))->pluck('company_id')->first();

        $company = Company::find($company_id);

        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'sales_invoice'])->first();

        return view('crusher.trans_sale_invoice.view', compact('salesInvoice', 'company', 'tnc'));
    }


    public function edit(AdminSalesInvoice $adminSalesInvoice)
    {
        // Retrieve permission titles from Permission table
        $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        // Filter permission titles using hasBranchAccountPermission function
        $filteredPermissionTitles = [];
        $branchId = session()->get('selected_branch');
        foreach ($permissionTitles as $permissionTitle) {
            if (Branches::hasBranchAccountPermission($branchId, $permissionTitle)) {
                $filteredPermissionTitles[] = $permissionTitle;
            }
        }

        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'6'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'6'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'sales_invoice'])->first();

        //  // Retrieve customers and include both 'account_name' and 'table_name' attributes
        //  $customers = Customer::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        //  ->select('*', 'name_of_organisation as account_name')
        //  ->get()
        //  ->map(function ($item) {
        //      return $item->setAttribute('table_name', 'App\Customer');
        //  });

        //  // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        //  $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        //  ->select('*', 'name_of_organisation as account_name')
        //  ->get()
        //  ->map(function ($item) {
        //      return $item->setAttribute('table_name', 'App\Supplier');
        //  });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        // ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        // ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        // });

        //  // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        //  $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        //  ->get()
        //  ->map(function ($item) {
        //      return $item->setAttribute('table_name', 'App\Expense');
        //  });

        //  // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        //  $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        //  ->get()
        //  ->map(function ($item) {
        //      return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        //  });

        $userId = Auth::id();

        $branchId = session('selected_branch');

        $permissionIds = AccountMasterPermission::where([
            ['user_id', $userId],
            ['branch_id', $branchId],
        ])->pluck('account_master_no');

        // whereIn('acc_no', $permissionIds)

        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) AS account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->whereIn('account_no', $permissionIds)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            $item->setAttribute('table_name', 'App\AdminOtherAccount');
            $item->setAttribute('class_of_account', ($item->class_of_account_id == 1 || $item->class_of_account_id == 3) ? '1,3' : '');
            return $item;
        });


         // Concatenate all collections into a single collection
         $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);


        //  $salesInvoices = $adminSalesInvoice->with('account', 'item', 'warehouse', 'expenses')->where('invoice_no', $adminSalesInvoice->invoice_no)->get();

        $salesInvoices = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
        ->with(['account', 'item', 'warehouse', 'expenses'])
        ->where('invoice_no', $adminSalesInvoice->invoice_no)
        ->get();

        $periodClosings = AdminPeriodClosing::where('branch_id', session()->get('selected_branch'))->pluck('to')->toArray();

        return view('crusher.trans_sale_invoice.edit', compact('tnc', 'items', 'warehouses', 'expensesAccount', 'accounts', 'salesInvoices','periodClosings'));
    }

    public function update(Request $request, AdminSalesInvoice $adminSalesInvoice)
    {
        // dd($request);
        try {
            $validatedData = $request->validate([
                'invoice_no' => 'nullable|string',
                'quotation_no' => 'nullable|string',
                'do_no' => 'nullable|string',
                'lpo_no' => 'nullable|string',
                'lpo_days' => 'nullable|string',
                'term_of_payment' => 'nullable|string',
                'narrations' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'account_type' => 'nullable',
                'name_account' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'item_id' => 'nullable|array',
                'item_id.*' => 'nullable|exists:item_names,id',
                'warehouse_id' => 'nullable|array',
                'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'item_unit' => 'nullable|array',
                'item_unit.*' => 'nullable|string',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'expense_type' => 'nullable|array',
                'expense_type.*' => 'nullable',
                'account_name' => 'nullable|array',
                'account_name.*' => 'nullable',
                'acc_no' => 'nullable',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable|numeric',
                'total_net_amount' => 'nullable|numeric',
                'discount' => 'nullable',
                'AmountInWords' => 'nullable|string',
            ]);

            $branch_id = session()->get('selected_branch');

            $adminSalesInvoice->with('account', 'item', 'warehouse', 'expenses')->where('branch_id', session()->get('selected_branch'))->where('invoice_no', $adminSalesInvoice->invoice_no)->forceDelete();

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $invoiceData = [
                    'invoice_no' => $validatedData['invoice_no'],
                    'quotation_no' => $validatedData['quotation_no'],
                    'do_no' => $validatedData['do_no'],
                    'lpo_no' => $validatedData['lpo_no'],
                    'lpo_days' => $validatedData['lpo_days'],
                    'term_of_payment' => $validatedData['term_of_payment'],
                    'narrations' => $validatedData['narrations'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'account_name' => $validatedData['name_account'],
                    'acc_no' => $validatedData['acc_no'],
                    'item_description' => $validatedData['item_description'][$i] ?? null,
                    'quantity' => $validatedData['quantity'][$i] ?? null,
                    'rate_unit' => $validatedData['rate_unit'][$i] ?? null,
                    'item_unit' => $validatedData['item_unit'][$i] ?? null,
                    'gross_amount' => $validatedData['gross_amount'][$i] ?? null,
                    'vat_percentage' => $validatedData['vat_percentage'][$i] ?? null,
                    'vat_amount' => $validatedData['vat_amount'][$i] ?? null,
                    'net_amount' => $validatedData['net_amount'][$i] ?? null,
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'AmountInWords' => $validatedData['AmountInWords'],
                    'discount' => $validatedData['discount'],
                    'branch_id' => $branch_id,
                    'item_id' => $validatedData['item_id'][$i] ?? null,
                    'warehouse_id' => $validatedData['warehouse_id'][$i] ?? null,
                ];

                $invoice = AdminSalesInvoice::create($invoiceData);

                AdminSalesInvoiceExpense::where('admin_sales_invoice_id', $adminSalesInvoice->id)->forceDelete();
                // Add expenses for this invoice
                if (isset($validatedData['expense_id']) && count($validatedData['expense_id']) > 0) {
                    for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                        AdminSalesInvoiceExpense::create([
                            'admin_sales_invoice_id' => $invoice->id,
                            'expense_type' => $validatedData['expense_type'][$j] ?? null,
                            'account_name' => $validatedData['account_name'][$j] ?? null,
                            'acc_no' => $validatedData['acc_no'][$j] ?? null,
                            'expense_id' => $validatedData['expense_id'][$j] ?? null,
                            'expense_amount' => $validatedData['expense_amount'][$j] ?? null,
                        ]);
                    }
                }
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('software.admin_sales_invoice.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete($saleInvoice)
    {
        $saleInvoice = AdminSalesInvoice::findOrFail($saleInvoice);
        $saleInvoice->where('branch_id', session()->get('selected_branch'))->where('invoice_no', $saleInvoice->invoice_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('software.admin_sales_invoice.index');
    }

    function buyer(Request $request)
    {
        // return $request->all();
        try {
            unset($request['buyer_account_id']);
            unset($request['_token']);
            OtherAccountBuyer::firstOrCreate(['type'=>$request->type, 'invoice_no'=>$request->invoice_no],$request->all());
            return response()->json(['status'=>true]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerExists(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>$request->type, 'invoice_no'=>$request->invoice_no])->exists();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerDelete(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>$request->type, 'invoice_no'=>$request->invoice_no])->delete();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }
}
