<?php

namespace App\Http\Controllers;

use App\Branches;
use Illuminate\Http\Request;
use App\ReceivedVoucher;
use App\MarketingDashboardImage;
use Carbon\Carbon;
use App\AdminSalesInvoice;
use App\AdminPurchaesInvoice;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;

class CrusherController extends Controller
{
    public function index()
    {
        $dashboardImage = MarketingDashboardImage::find(1);
        // ADMIN
        $branches = Branches::with('company','permissions.permission')->whereHas('company', function($q) {$q->where('user_id', session()->get('uv_user_id'));})->get();
        // SUBADMIN
        if(auth()->user()->role=='subadmin')
        {
            $branches = auth()->user()->permissions->load('branchPermission.permission', 'branchPermission.branch');
            // $branches = $branches->pluck('branchPermission.branch')->unique();
            $branches = $branches->map(function ($branch) {
                return $branch->branchPermission->branch ?? null;
            })->filter()->unique();
        }
        // Get the ID of the currently selected branch from the session
        $selectedBranchId = session('selected_branch');
        $expire = false;
        $leniencyExpire = false;
        $status = false;
        $branch_status = false;

        if (!empty($selectedBranchId)) {
            $branch = Branches::find($selectedBranchId);

            if ($branch) {
                session()->put('branch', $branch);
                $branch_status = $branch->branch_status == 'deleted';
            }
        }

        if (!empty($selectedBranchId))
        {
            $branch = Branches::find($selectedBranchId);
            session()->put('branch',$branch);
            $now = Carbon::parse(now())->format('Y-m-d');
            $branchExpireDate = Carbon::parse($branch->expiry_date);
            $leniency = $branch->leniency;
            if($branchExpireDate->addDays($leniency)->lt($now))
            {
                $expire = true;
                if (!$branch->vouchers->isEmpty())
                {
                    foreach ($branch->vouchers as $voucher)
                    {
                        if ($voucher->created_at->gte($branchExpireDate))
                        {
                            $expire = false;
                        }
                    }
                }
            }

            $status = $branch->status;
        }

        // Find the selected branch within the fetched branches
        $selectedBranch = $branches->first(function ($branch) use ($selectedBranchId) {
            return $branch->id == $selectedBranchId;
        });

        $user = auth()->user();


        $chartData = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
        ->select(
            DB::raw('DATE_FORMAT(date, "%Y-%m") as month'),
            DB::raw('SUM(total_net_amount) as total_amount')
        )
        ->groupBy('month')
        ->get();


        $categories = $chartData->pluck('month')->toArray();
        $seriesData = $chartData->pluck('total_amount')->map(function ($value) {
            return number_format($value, 3, '.', '');
        })->toArray();

        $chartDataDayWise = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
        ->select(
            DB::raw('DATE_FORMAT(date, "%Y-%m-%d") as day'),
            DB::raw('SUM(total_net_amount) as total_amount')
        )
        ->groupBy('day')
        ->get();


        $chartDataLast7Days = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
            ->where('date', '>=', now()->subDays(7)->format('Y-m-d'))
            ->select(
                DB::raw('DATE_FORMAT(date, "%Y-%m-%d") as day'),
                DB::raw('SUM(total_net_amount) as total_amount')
            )
            ->groupBy('day')
            ->get();

        $chartDataPurchase = AdminPurchaesInvoice::where('branch_id', session()->get('selected_branch'))
        ->select(
            DB::raw('DATE_FORMAT(date, "%Y-%m") as month'),
            DB::raw('SUM(total_net_amount) as total_amount')
        )
        ->groupBy('month')
        ->get();

        $categoriesPurchase = $chartDataPurchase->pluck('month')->toArray();
        $seriesDataPurchase = $chartDataPurchase->pluck('total_amount')->map(function ($value) {
            return number_format($value, 3, '.', '');
        })->toArray();

        // // Get the date 30 days ago from today
        // $date = Carbon::now()->subDays(30);

        // // Query to get the data from the last 30 days and calculate the sum
        // $last30DaysTotalAmount = DB::table('admin_sales_invoices')
        //                     ->where('branch_id', session()->get('selected_branch'))
        //                     ->where('date', '>=', $date)
        //                     ->sum('total_net_amount');

                            // dd($last30DaysTotalAmount);

        // dd($categoriesPurchase,$seriesDataPurchase);

        return view('crusher.index',compact('dashboardImage', 'branches', 'selectedBranch', 'user', 'expire', 'status','branch_status','chartData','categories','seriesData','chartDataDayWise','chartDataLast7Days','categoriesPurchase','seriesDataPurchase','chartDataPurchase'));
    }

    public function updateTermsAccept(Request $request)
    {
        // Get the authenticated user (you may need to modify this depending on your authentication setup)
        $user = Auth::user();

        // Update the terms_accept status to 1
        $user->update(['terms_accept' => 1]);

        return redirect()->back()->with('success', 'Terms and conditions accepted.');
    }

    public function upload(Request $request)
    {
        if ($request->hasFile('voucher')) {
            $file = $request->file('voucher');

            $fileName = time() . '-voucher.' . $file->getClientOriginalExtension();

            $destinationPath = public_path('uploads/branch/vouchers');

            $file->move($destinationPath, $fileName);

            $filePath = 'uploads/branch/vouchers/' . $fileName;

            $data = [
                'branch_id' => session('selected_branch'),
                'voucher' => $filePath
            ];

            ReceivedVoucher::create($data);

            flash()->addSuccess('Uploaded');
            return back();
        } else {
            flash()->addError('Voucher is required');
            return back();
        }
    }

    public function setSelectedBranch(Request $request)
    {
        $branchId = $request->input('branch');

        if ($branchId) {
            Session::put('selected_branch', $branchId);

        } else {
            Session::forget('selected_branch');
        }

        // dd(session()->get('selected_branch'));

        return redirect()->back();
    }
}
