<?php

namespace App\Http\Controllers;
use App\Branches;
use App\ClassOfAccount;
use App\Country;
use App\Customer;
use App\AccountMasterPermission;
use App\User;
use App\Http\Requests\StoreCustomerRequest;
use App\Http\Requests\UpdateCustomerRequest;
use Illuminate\Support\Facades\Storage;
use App\Permission;
use App\BranchPermission;
use App\AssignedPermission;
use Auth;

class CustomerMasterController extends Controller
{
    public function index()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'customers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);

        $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        // $filteredPermissionTitles = [];
        // $branchId = session()->get('selected_branch');
        // foreach ($permissionTitles as $permissionTitle) {
        //     if (Branches::hasBranchAccountPermission($branchId, $permissionTitle)) {
        //         $filteredPermissionTitles[] = $permissionTitle;
        //     }
        // }

        // $customers = Customer::where('branch_id', $branchId)
        //     ->orderBy('id', 'desc')
        //     ->get();

        // // Modify the values of name_of_organisation column in customers
        // foreach ($customers as $customer) {
        //     $customer->name_of_organisation = str_replace(' ', '-', $customer->name_of_organisation);
        // }

        // // Filter customers based on modified name_of_organisation column
        // $filteredCustomers = $customers->filter(function ($customer) use ($filteredPermissionTitles) {
        //     return in_array($customer->name_of_organisation, $filteredPermissionTitles);
        // });

        // You can use $filteredCustomers as needed

        $userId = Auth::id();
        // dd($userId);
        $branchId = session('selected_branch');

        $customerIds = AccountMasterPermission::where([
            ['user_id', $userId],
            ['branch_id', $branchId],
            ['account_master_type', 'customer']
        ])->pluck('account_master_id');

        $customers = Customer::whereIn('id', $customerIds)->where('branch_id', $branchId)
            ->orderBy('id', 'desc')
            ->get();



        return view('crusher.customer_master.index', compact('customers'));
    }

    public function create()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'customers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);

        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        // $maintainAccount = Maintain::all();

        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        // dd($subadmins);

        return view('crusher.customer_master.create', compact('countries', 'classOfAccount','subadmins'));
    }

    public function store(StoreCustomerRequest $request)
    {

        // dd($request);

        $masterName = 'customer';

        $latestAccount = Customer::withTrashed()->where('branch_id', session()->get('selected_branch'))
            ->latest('account_no')->first();

        if ($latestAccount) {
            $explodedAccountNo = explode('-', $latestAccount->account_no);
            $lastAccountNumber = end($explodedAccountNo);
            $newAccountNumber = $masterName . '-' . str_pad((int)$lastAccountNumber + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newAccountNumber = $masterName . '-0001';
        }

        // dd($newAccountNumber);

        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'customers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        try {

            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');
            $validatedData['account_no'] = $newAccountNumber;

            // dd($validatedData);

            for ($i = 1; $i <= 3; $i++) {
                if ($request->hasFile("authorised_sign_{$i}_sign_sample")) {
                    $file = $request->file("authorised_sign_{$i}_sign_sample");
                    $fileName = time() . '_auth_sign_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/customer_master/authorised_docs/', $fileName, 'public');
                    $validatedData["authorised_sign_{$i}_sign_sample"] = $filePath;
                }
            }

            // Handle file uploads for documents
            for ($i = 1; $i <= 7; $i++) {
                if ($request->hasFile("doc_file_$i")) {
                    $file = $request->file("doc_file_$i");
                    $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/customer_master/docs/', $fileName, 'public');
                    $validatedData["doc_file_$i"] = $filePath;
                }
            }


            $name = str_replace(' ', '-', $request->name_of_organisation);
            $selectedBranchId = session('selected_branch');

            // $permission = Permission::create([
            //     'name' => $name,
            //     'guard_name' => 'web',
            //     'type' => 'Ledger Customer',
            //     'branch_id' => $selectedBranchId,
            // ]);

            // $BranchPermission = BranchPermission::create([
            //     'branch_id' => $permission->branch_id,
            //     'permission_id' => $permission->id,
            // ]);

            // if (auth()->check() && auth()->user()->role == 'subadmin') {
            //     $subadminId = auth()->user()->id;

            //     AssignedPermission::create([
            //         'subadmin_id' => $subadminId,
            //         'branch_permission_id' => $BranchPermission->id
            //     ]);
            // } else {

            // }

            // dd($permission);

        // Create a new customer record
        $customer = Customer::create($validatedData);

        $current_customer_id = $customer->id;
        $current_customer_account_no = $customer->account_no;

        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = $request->show_user;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_customer_id;
                $account_master_permission->account_master_no = $current_customer_account_no;
                $account_master_permission->account_master_type = "customer";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }
        }
        if(Auth::check() && Auth::user()->role === 'admin')
        {
            $account_master_permission = new AccountMasterPermission();
            $account_master_permission->user_id = Auth::user()->id;
            $account_master_permission->branch_id = $selectedBranchId;
            $account_master_permission->account_master_id = $current_customer_id;
            $account_master_permission->account_master_no = $current_customer_account_no;
            $account_master_permission->account_master_type = "customer";
            $account_master_permission->account_master_name = $name;
            $account_master_permission->save();
        }else{
            $account_master_permission = new AccountMasterPermission();
            $account_master_permission->user_id = Auth::user()->id;
            $account_master_permission->branch_id = $selectedBranchId;
            $account_master_permission->account_master_id = $current_customer_id;
            $account_master_permission->account_master_no = $current_customer_account_no;
            $account_master_permission->account_master_type = "customer";
            $account_master_permission->account_master_name = $name;
            $account_master_permission->save();

            $account_master_permission = new AccountMasterPermission();
            $account_master_permission->user_id = Auth::user()->admin_id;
            $account_master_permission->branch_id = $selectedBranchId;
            $account_master_permission->account_master_id = $current_customer_id;
            $account_master_permission->account_master_no = $current_customer_account_no;
            $account_master_permission->account_master_type = "customer";
            $account_master_permission->account_master_name = $name;
            $account_master_permission->save();
        }


            flash()->addSuccess('Record Created');
            return redirect()->route('software.customers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function edit($customer)
    {
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        $branchId = session('selected_branch');

        $account_master_permission = AccountMasterPermission::where([
            ['branch_id', $branchId],
            ['account_master_id', $customer],
            ['account_master_type', 'customer']
        ])->pluck('user_id');

        // dd($account_master_permission);

        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'customers-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        $customer = Customer::findOrFail($customer);



        return view('crusher.customer_master.edit', compact('countries', 'classOfAccount', 'customer','account_master_permission','subadmins'));
    }

    public function show($customer)
    {
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        $branchId = session('selected_branch');

        $account_master_permission = AccountMasterPermission::where([
            ['branch_id', $branchId],
            ['account_master_id', $customer],
            ['account_master_type', 'customer']
        ])->pluck('user_id');

        // dd($account_master_permission);

        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'customers-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        $customer = Customer::with('country', 'classOfAccount')->findOrFail($customer);

       

        return view('crusher.customer_master.view', compact('countries', 'classOfAccount', 'customer','subadmins','account_master_permission'));
    }

    public function update(UpdateCustomerRequest $request, Customer $customer)
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'customers-account-master')) {
            return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        }

        try {
            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');

            // Handle file uploads for authorized signatory signature samples
            for ($i = 1; $i <= 3; $i++) {
                if ($request->hasFile("authorised_sign_{$i}_sign_sample")) {
                    $file = $request->file("authorised_sign_{$i}_sign_sample");
                    $fileName = time() . '_auth_sign_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/customer_master/authorised_docs/', $fileName, 'public');
                    $validatedData["authorised_sign_{$i}_sign_sample"] = $filePath;

                    // Delete existing file if it exists
                    if (!empty($customer["authorised_sign_{$i}_sign_sample"])) {
                        Storage::disk('public')->delete($customer["authorised_sign_{$i}_sign_sample"]);
                    }
                }
            }

            // Handle file uploads for documents
            for ($i = 1; $i <= 7; $i++) {
                if ($request->hasFile("doc_file_$i")) {
                    $file = $request->file("doc_file_$i");
                    $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/customer_master/docs/', $fileName, 'public');
                    $validatedData["doc_file_$i"] = $filePath;

                    // Delete existing file if it exists
                    if (!empty($customer["doc_file_$i"])) {
                        Storage::disk('public')->delete($customer["doc_file_$i"]);
                    }
                }
            }

            // Update the customer record
            $customer->update($validatedData);

            $name = str_replace(' ', '-', $request->name_of_organisation);
            $current_customer_id = $customer->id;
            $current_customer_account_no = $customer->account_no;


            $selectedBranchId = session('selected_branch');

            $account_master_permission = AccountMasterPermission::where([
                ['branch_id', $branchId],
                ['account_master_id', $current_customer_id],
                ['account_master_type', 'customer']
            ])->get();

            foreach ($account_master_permission as $item) {
                    $item->delete();
            }

            if(isset($request->show_user)){
                foreach ($request->show_user as $key => $request->show_user) {
                    $account_master_permission = new AccountMasterPermission();
                    $account_master_permission->user_id = $request->show_user;
                    $account_master_permission->branch_id = $selectedBranchId;
                    $account_master_permission->account_master_id = $current_customer_id;
                    $account_master_permission->account_master_no = $current_customer_account_no;
                    $account_master_permission->account_master_type = "customer";
                    $account_master_permission->account_master_name = $name;
                    $account_master_permission->save();
                }
            }
            if(Auth::check() && Auth::user()->role === 'admin')
            {
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_customer_id;
                $account_master_permission->account_master_no = $current_customer_account_no;
                $account_master_permission->account_master_type = "customer";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }else{
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_customer_id;
                $account_master_permission->account_master_no = $current_customer_account_no;
                $account_master_permission->account_master_type = "customer";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();

                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->admin_id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_customer_id;
                $account_master_permission->account_master_no = $current_customer_account_no;
                $account_master_permission->account_master_type = "customer";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('software.customers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function softDelete($customer)
    {
        $customer = Customer::findOrFail($customer);
        $customer->delete();
        flash()->addSuccess('Success');
        return redirect()->route('software.customers.index');
    }
}
