<?php

namespace App\Http\Controllers;

use App\AssignedPermission;
use App\Branches;
use Illuminate\Http\Request;
use App\Notifications;
use App\NotificationTrack;
use App\User;
use App\UserRemainder;

class NotificationsController extends Controller
{
    public function index()
    {
        $notifications = Notifications::where('branch_id',session('selected_branch'))->get();
        return view('crusher.notifications.crusher-notification', compact('notifications'));
    }

    function branches()
    {
        try {
            $branches = Branches::where(['country'=>request()->country, 'company_id'=>auth()->user()->company->id])->get();
            return ['status'=>true, 'data'=>$branches];
        } catch (\Throwable $th) {
            return ['status'=>false, 'data'=>$th->getMessage()];
        }
    }

    public function create()
    {
        $branches = Branches::whereHas('company', function($q){$q->where('user_id', auth()->id());})->get();
        return view('crusher.notifications.crusher-add-notification', compact('branches'));
    }

    function branchSubadmins()
    {
        $userIds = AssignedPermission::whereHas('branchPermission', function($q){$q->where('branch_id', request()->branch);})->pluck('subadmin_id')->unique();
        $users = User::whereIn('id', $userIds)->get();
        return $users;
    }

    function store(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'title' => 'nullable|string',
                'description' => 'nullable|string',
            ]);
            $validatedData['branch_id'] = session()->get('selected_branch');
            if (request()->sendToAll) {
                $notification =  Notifications::create($validatedData);
               foreach (auth()->user()->users as $user) {
                   NotificationTrack::create(['notification_id'=>$notification->id, 'user_id'=>$user->id]);
               }
               flash()->addSuccess('Sent');
               return redirect()->route('crusher.notifications.index');
            }
            if (request()->sendToBranch) {
                $notification =  Notifications::create($validatedData);
                foreach (request()->users as $user) {
                    NotificationTrack::create(['notification_id'=>$notification->id, 'user_id'=>$user]);
                }
                flash()->addSuccess('Sent');
                return redirect()->route('software.notifications.index');
            }
            flash()->addError('No option selected');
            return back();
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show()
    {
        $notification = Notifications::find(request()->notification);
        return view('crusher.notifications.crusher-show-notification', compact('notification'));
    }

    public function softDelete($notification)
    {
        $notification = Notifications::findOrFail($notification);
        $notification->delete();
        flash()->addSuccess('Success');
        return back();
    }

    function updateViewed()
    {
        try {
            NotificationTrack::where('user_id',auth()->id())->update(['is_viewed'=>1]);
            UserRemainder::where('user_id',auth()->id())->update(['is_viewed'=>1]);
        return true;
        } catch (\Throwable $th) {
            return $th->getMessage();
        }

    }
}
