<?php

namespace App\Http\Controllers;

use App\Branches;
use App\ClassOfAccount;
use App\Country;
use App\Http\Requests\StoreStaffRequest;
use App\Http\Requests\UpdateStaffRequest;
use App\StaffAndSalesman;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\DB;
use App\Permission;
use App\BranchPermission;
use App\AssignedPermission;
use App\User;
use App\AccountMasterPermission;
use Auth;

class StaffMasterController extends Controller
{
    public function index()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        $staffs = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))->orderBy('id', 'desc')->get();

        //   // Retrieve permission titles from Permission table
        //   $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        //   // Filter permission titles using hasBranchPermission function
        //   $filteredPermissionTitles = [];
        //   $branchId = session()->get('selected_branch');
        //   foreach ($permissionTitles as $permissionTitle) {
        //       if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
        //           $filteredPermissionTitles[] = $permissionTitle;
        //       }
        //   }

        //   // Retrieve StaffAndSalesman whose first_name last_name matches filtered permission titles
        //   $staffs = StaffAndSalesman::where('branch_id', $branchId)
        //     ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        //     ->orderBy('id', 'desc')
        //     ->get();




        // Retrieve permission titles from Permission table
        // $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        // // Filter permission titles using hasBranchPermission function
        // $filteredPermissionTitles = [];
        // $branchId = session()->get('selected_branch');
        // foreach ($permissionTitles as $permissionTitle) {
        //     if (Branches::hasBranchAccountPermission($branchId, $permissionTitle)) {
        //         $filteredPermissionTitles[] = $permissionTitle;
        //     }
        // }

        // // Retrieve StaffAndSalesman whose first_name last_name matches filtered permission titles
        // $staffs = StaffAndSalesman::where('branch_id', $branchId)
        //     ->orderBy('id', 'desc')
        //     ->get();

        // // Modify the values of first_name and last_name columns in staffs
        // foreach ($staffs as $staff) {
        //     $firstName = str_replace(' ', '-', $staff->first_name);
        //     $lastName = str_replace(' ', '-', $staff->last_name);
        //     $staff->full_name = $firstName . '-' . $lastName;
        // }

        // // Filter staffs based on modified full_name column
        // $filteredStaffs = $staffs->filter(function ($staff) use ($filteredPermissionTitles) {
        //     return in_array($staff->full_name, $filteredPermissionTitles);
        // });

        // You can use $filteredStaffs as needed

        $userId = Auth::id();
        // dd($userId);
        $branchId = session('selected_branch');

        $staffsIds = AccountMasterPermission::where([
            ['user_id', $userId],
            ['branch_id', $branchId],
            ['account_master_type', 'staffAndSalesman']
        ])->pluck('account_master_id');

        $staffs = StaffAndSalesman::whereIn('id', $staffsIds)->where('branch_id', $branchId)
            ->orderBy('id', 'desc')
            ->get();


        return view('crusher.staff_master.index', compact('staffs'));
    }

    public function create()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);

        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        // $maintainAccount = Maintain::all();
        return view('crusher.staff_master.create', compact('countries', 'classOfAccount','subadmins'));
    }

    public function store(StoreStaffRequest $request)
    {
        $masterName = 'staff-and-salesmen';
        $latestAccount = StaffAndSalesman::withTrashed()->where('branch_id', session()->get('selected_branch'))
            ->latest('account_no')->first();

        if ($latestAccount) {
            $explodedAccountNo = explode('-', $latestAccount->account_no);
            $lastAccountNumber = end($explodedAccountNo);
            $newAccountNumber = $masterName . '-' . str_pad((int)$lastAccountNumber + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newAccountNumber = $masterName . '-0001';
        }

        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        try {

            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');
            $validatedData['account_no'] = $newAccountNumber;

            // Handle file uploads for documents
            for ($i = 1; $i <= 7; $i++) {
                if ($request->hasFile("doc_file_$i")) {
                    $file = $request->file("doc_file_$i");
                    $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/staff_master/docs/', $fileName, 'public');
                    $validatedData["doc_file_$i"] = $filePath;
                }
            }

        $staffAndSalesman = StaffAndSalesman::create($validatedData);

        $first_name = str_replace(' ', '-', $request->first_name);
        $last_name = str_replace(' ', '-', $request->last_name);
        $name = $first_name . '-' . $last_name;
        $selectedBranchId = session('selected_branch');

            $current_staffAndSalesman_id = $staffAndSalesman->id;
            $current_staffAndSalesman_no = $staffAndSalesman->account_no;

            if(isset($request->show_user)){
                foreach ($request->show_user as $key => $request->show_user) {
                    $account_master_permission = new AccountMasterPermission();
                    $account_master_permission->user_id = $request->show_user;
                    $account_master_permission->branch_id = $selectedBranchId;
                    $account_master_permission->account_master_id = $current_staffAndSalesman_id;
                    $account_master_permission->account_master_no = $current_staffAndSalesman_no;
                    $account_master_permission->account_master_type = "staffAndSalesman";
                    $account_master_permission->account_master_name = $name;
                    $account_master_permission->save();
                }
            }

            if(Auth::check() && Auth::user()->role === 'admin')
            {
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_staffAndSalesman_id;
                $account_master_permission->account_master_no = $current_staffAndSalesman_no;
                $account_master_permission->account_master_type = "staffAndSalesman";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }else{
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_staffAndSalesman_id;
                $account_master_permission->account_master_no = $current_staffAndSalesman_no;
                $account_master_permission->account_master_type = "staffAndSalesman";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();

                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->admin_id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_staffAndSalesman_id;
                $account_master_permission->account_master_no = $current_staffAndSalesman_no;
                $account_master_permission->account_master_type = "staffAndSalesman";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }

        // $permission = Permission::create([
        //     'name' => $name,
        //     'guard_name' => 'web',
        //     'type' => 'Ledger Staff and Salesman',
        //     'branch_id' => $selectedBranchId,
        // ]);

        // $BranchPermission = BranchPermission::create([
        //     'branch_id' => $permission->branch_id,
        //     'permission_id' => $permission->id,
        // ]);

        // if (auth()->check() && auth()->user()->role == 'subadmin') {
        //     $subadminId = auth()->user()->id;

        //     AssignedPermission::create([
        //         'subadmin_id' => $subadminId,
        //         'branch_permission_id' => $BranchPermission->id
        //     ]);
        // } else {

        // }

            flash()->addSuccess('Record Created');
            return redirect()->route('software.staffs.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function edit($staff)
    {
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        $branchId = session('selected_branch');

        $account_master_permission = AccountMasterPermission::where([
            ['branch_id', $branchId],
            ['account_master_id', $staff],
            ['account_master_type', 'staffAndSalesman']
        ])->pluck('user_id');

        // dd($account_master_permission);

        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        $staff = StaffAndSalesman::findOrFail($staff);
        return view('crusher.staff_master.edit', compact('countries', 'classOfAccount', 'staff','subadmins','account_master_permission'));
    }

    public function update(UpdateStaffRequest $request, StaffAndSalesman $staff)
    {
        $current_staffAndSalesman_id = $staff->id;

        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) {
            return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        }

        try {
            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');

            // Handle file uploads for authorized signatory signature samples
            for ($i = 1; $i <= 3; $i++) {
                if ($request->hasFile("authorised_sign_{$i}_sign_sample")) {
                    $file = $request->file("authorised_sign_{$i}_sign_sample");
                    $fileName = time() . '_auth_sign_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/staff_master/authorised_docs/', $fileName, 'public');
                    $validatedData["authorised_sign_{$i}_sign_sample"] = $filePath;

                    // Delete existing file if it exists
                    if (!empty($staff["authorised_sign_{$i}_sign_sample"])) {
                        Storage::disk('public')->delete($staff["authorised_sign_{$i}_sign_sample"]);
                    }
                }
            }

            // Handle file uploads for documents
            for ($i = 1; $i <= 7; $i++) {
                if ($request->hasFile("doc_file_$i")) {
                    $file = $request->file("doc_file_$i");
                    $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/staff_master/docs/', $fileName, 'public');
                    $validatedData["doc_file_$i"] = $filePath;

                    // Delete existing file if it exists
                    if (!empty($staff["doc_file_$i"])) {
                        Storage::disk('public')->delete($staff["doc_file_$i"]);
                    }
                }
            }

            // Update the staff record
            $staff->update($validatedData);

            $first_name = str_replace(' ', '-', $request->first_name);
            $last_name = str_replace(' ', '-', $request->last_name);
            $name = $first_name . '-' . $last_name;
            $selectedBranchId = session('selected_branch');
            $current_staffAndSalesman_no = $request->account_no;

            $account_master_permission = AccountMasterPermission::where([
                ['branch_id', $selectedBranchId],
                ['account_master_id', $current_staffAndSalesman_id],
                ['account_master_type', 'staffAndSalesman']
            ])->get();

            foreach ($account_master_permission as $item) {
                    $item->delete();
            }

            if(isset($request->show_user)){
                foreach ($request->show_user as $key => $request->show_user) {
                    $account_master_permission = new AccountMasterPermission();
                    $account_master_permission->user_id = $request->show_user;
                    $account_master_permission->branch_id = $selectedBranchId;
                    $account_master_permission->account_master_id = $current_staffAndSalesman_id;
                    $account_master_permission->account_master_no = $current_staffAndSalesman_no;
                    $account_master_permission->account_master_type = "staffAndSalesman";
                    $account_master_permission->account_master_name = $name;
                    $account_master_permission->save();
                }
            }

            if(Auth::check() && Auth::user()->role === 'admin')
            {
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_staffAndSalesman_id;
                $account_master_permission->account_master_no = $current_staffAndSalesman_no;
                $account_master_permission->account_master_type = "staffAndSalesman";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }else{
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_staffAndSalesman_id;
                $account_master_permission->account_master_no = $current_staffAndSalesman_no;
                $account_master_permission->account_master_type = "staffAndSalesman";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();

                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->admin_id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_staffAndSalesman_id;
                $account_master_permission->account_master_no = $current_staffAndSalesman_no;
                $account_master_permission->account_master_type = "staffAndSalesman";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('software.staffs.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function softDelete($staff)
    {
        $staff = StaffAndSalesman::findOrFail($staff);
        $staff->delete();
        flash()->addSuccess('Success');
        return redirect()->route('software.staffs.index');
    }

    function show($staff)
    {
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        $branchId = session('selected_branch');

        $account_master_permission = AccountMasterPermission::where([
            ['branch_id', $branchId],
            ['account_master_id', $staff],
            ['account_master_type', 'staffAndSalesman']
        ])->pluck('user_id');

        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'staff-and-salesmen-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $staff = StaffAndSalesman::with('country', 'classOfAccount')->findOrFail($staff);
        return view('crusher.staff_master.view', compact('staff','subadmins','account_master_permission'));
    }
}
