<?php

namespace App\Http\Controllers;

use App\Branches;
use App\ClassOfAccount;
use App\Country;
use App\Supplier;
use App\Http\Requests\StoreSupplierRequest;
use App\Http\Requests\UpdateSupplierRequest;
use Illuminate\Support\Facades\Storage;
use App\Permission;
use App\BranchPermission;
use App\AssignedPermission;
use App\User;
use App\AccountMasterPermission;
use Auth;

class SupplierMasterController extends Controller
{
    // public function index()
    // {
    //     if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
    //     $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))->orderBy('id', 'desc')->get();

    //     return view('crusher.supplier_master.index', compact('suppliers'));
    // }

    public function index()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);

        // // Retrieve permission titles from Permission table
        // $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        // // Filter permission titles using hasBranchPermission function
        // $filteredPermissionTitles = [];
        // $branchId = session()->get('selected_branch');
        // foreach ($permissionTitles as $permissionTitle) {
        //     if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
        //         $filteredPermissionTitles[] = $permissionTitle;
        //     }
        // }

        // // Retrieve suppliers whose name_of_organisation matches filtered permission titles
        // $suppliers = Supplier::where('branch_id', $branchId)
        //     ->whereIn('name_of_organisation', $filteredPermissionTitles)
        //     ->orderBy('id', 'desc')
        //     ->get();

        // $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        // $filteredPermissionTitles = [];
        // $branchId = session()->get('selected_branch');
        // foreach ($permissionTitles as $permissionTitle) {
        //     if (Branches::hasBranchAccountPermission($branchId, $permissionTitle)) {
        //         $filteredPermissionTitles[] = $permissionTitle;
        //     }
        // }

        // $suppliers = Supplier::where('branch_id', $branchId)
        //     ->orderBy('id', 'desc')
        //     ->get();

        // foreach ($suppliers as $supplier) {
        //     $supplier->account_name = str_replace(' ', '-', $supplier->account_name);
        // }

        // $filteredSuppliers = $suppliers->filter(function ($supplier) use ($filteredPermissionTitles) {
        //     return in_array($supplier->account_name, $filteredPermissionTitles);
        // });

        $userId = Auth::id();
        // dd($userId);
        $branchId = session('selected_branch');

        $suppliersIds = AccountMasterPermission::where([
            ['user_id', $userId],
            ['branch_id', $branchId],
            ['account_master_type', 'supplier']
        ])->pluck('account_master_id');

        $suppliers = Supplier::whereIn('id', $suppliersIds)->where('branch_id', $branchId)
            ->orderBy('id', 'desc')
            ->get();

        return view('crusher.supplier_master.index', compact('suppliers'));
    }

    public function create()
    {
        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);

        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        // $maintainAccount = Maintain::all();
        return view('crusher.supplier_master.create', compact('countries', 'classOfAccount','subadmins'));
    }

    public function store(StoreSupplierRequest $request)
    {
        $masterName = 'supplier';
        $latestAccount = Supplier::withTrashed()->where('branch_id', session()->get('selected_branch'))
            ->latest('account_no')->first();

        if ($latestAccount) {
            $explodedAccountNo = explode('-', $latestAccount->account_no);
            $lastAccountNumber = end($explodedAccountNo);
            $newAccountNumber = $masterName . '-' . str_pad((int)$lastAccountNumber + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newAccountNumber = $masterName . '-0001';
        }

        if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
        try {

            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');
            $validatedData['account_no'] = $newAccountNumber;

            // Handle file uploads for documents
            for ($i = 1; $i <= 7; $i++) {
                if ($request->hasFile("doc_file_$i")) {
                    $file = $request->file("doc_file_$i");
                    $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                    $filePath = $file->storeAs('uploads/admin/supplier_master/docs/', $fileName, 'public');
                    $validatedData["doc_file_$i"] = $filePath;
                }
            }

        $supplier = Supplier::create($validatedData);


            $name = str_replace(' ', '-', $request->name_of_organisation);
            $selectedBranchId = session('selected_branch');

            $current_supplier_id = $supplier->id;
            $current_supplier_no = $supplier->account_no;

            if(isset($request->show_user)){
                foreach ($request->show_user as $key => $request->show_user) {
                    $account_master_permission = new AccountMasterPermission();
                    $account_master_permission->user_id = $request->show_user;
                    $account_master_permission->branch_id = $selectedBranchId;
                    $account_master_permission->account_master_id = $current_supplier_id;
                    $account_master_permission->account_master_no = $current_supplier_no;
                    $account_master_permission->account_master_type = "supplier";
                    $account_master_permission->account_master_name = $name;
                    $account_master_permission->save();
                }
            }

            if(Auth::check() && Auth::user()->role === 'admin')
            {
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_supplier_id;
                $account_master_permission->account_master_no = $current_supplier_no;
                $account_master_permission->account_master_type = "supplier";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }else{
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_supplier_id;
                $account_master_permission->account_master_no = $current_supplier_no;
                $account_master_permission->account_master_type = "supplier";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();

                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = Auth::user()->admin_id;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_supplier_id;
                $account_master_permission->account_master_no = $current_supplier_no;
                $account_master_permission->account_master_type = "supplier";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }

            // $permission = Permission::create([
            //     'name' => $name,
            //     'guard_name' => 'web',
            //     'type' => 'Ledger Supplier',
            //     'branch_id' => $selectedBranchId,
            // ]);

            // $BranchPermission = BranchPermission::create([
            //     'branch_id' => $permission->branch_id,
            //     'permission_id' => $permission->id,
            // ]);

            // if (auth()->check() && auth()->user()->role == 'subadmin') {
            //     $subadminId = auth()->user()->id;

            //     AssignedPermission::create([
            //         'subadmin_id' => $subadminId,
            //         'branch_permission_id' => $BranchPermission->id
            //     ]);
            // } else {

            // }


            flash()->addSuccess('Record Created');
            return redirect()->route('software.suppliers.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function edit($supplier)
    {
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        $branchId = session('selected_branch');

        $account_master_permission = AccountMasterPermission::where([
            ['branch_id', $branchId],
            ['account_master_id', $supplier],
            ['account_master_type', 'supplier']
        ])->pluck('user_id');

        // dd($account_master_permission);

        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $countries = Country::all();
        $classOfAccount = ClassOfAccount::all();
        $supplier = Supplier::findOrFail($supplier);
        return view('crusher.supplier_master.edit', compact('countries', 'classOfAccount', 'supplier','subadmins','account_master_permission'));
    }

    public function update(UpdateSupplierRequest $request, Supplier $supplier)
{
    $current_supplier_id = $supplier->id;

    if (!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'suppliers-account-master')) {
        return response()->json(['status' => false, 'error' => 'Unauthorized'], 403);
    }

    try {
        $validatedData = $request->validated();
        $validatedData['branch_id'] = session()->get('selected_branch');

        // Handle file uploads for authorized signatory signature samples
        for ($i = 1; $i <= 3; $i++) {
            if ($request->hasFile("authorised_sign_{$i}_sign_sample")) {
                $file = $request->file("authorised_sign_{$i}_sign_sample");
                $fileName = time() . '_auth_sign_' . $i . '.' . $file->getClientOriginalExtension();
                $filePath = $file->storeAs('uploads/admin/supplier_master/authorised_docs/', $fileName, 'public');
                $validatedData["authorised_sign_{$i}_sign_sample"] = $filePath;

                // Delete existing file if it exists
                if (!empty($supplier["authorised_sign_{$i}_sign_sample"])) {
                    Storage::disk('public')->delete($supplier["authorised_sign_{$i}_sign_sample"]);
                }
            }
        }

        // Handle file uploads for documents
        for ($i = 1; $i <= 7; $i++) {
            if ($request->hasFile("doc_file_$i")) {
                $file = $request->file("doc_file_$i");
                $fileName = time() . '_doc_' . $i . '.' . $file->getClientOriginalExtension();
                $filePath = $file->storeAs('uploads/admin/supplier_master/docs/', $fileName, 'public');
                $validatedData["doc_file_$i"] = $filePath;

                // Delete existing file if it exists
                if (!empty($supplier["doc_file_$i"])) {
                    Storage::disk('public')->delete($supplier["doc_file_$i"]);
                }
            }
        }

        // Update the supplier record
        $supplier->update($validatedData);

        $name = str_replace(' ', '-', $request->name_of_organisation);
        $current_supplier_no = $request->account_no;

        $selectedBranchId = session('selected_branch');

        $account_master_permission = AccountMasterPermission::where([
            ['branch_id', $selectedBranchId],
            ['account_master_id', $current_supplier_id],
            ['account_master_type', 'supplier']
        ])->get();

        foreach ($account_master_permission as $item) {
                $item->delete();
        }

        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_permission = new AccountMasterPermission();
                $account_master_permission->user_id = $request->show_user;
                $account_master_permission->branch_id = $selectedBranchId;
                $account_master_permission->account_master_id = $current_supplier_id;
                $account_master_permission->account_master_no = $current_supplier_no;
                $account_master_permission->account_master_type = "supplier";
                $account_master_permission->account_master_name = $name;
                $account_master_permission->save();
            }
        }

        if(Auth::check() && Auth::user()->role === 'admin')
        {
            $account_master_permission = new AccountMasterPermission();
            $account_master_permission->user_id = Auth::user()->id;
            $account_master_permission->branch_id = $selectedBranchId;
            $account_master_permission->account_master_id = $current_supplier_id;
            $account_master_permission->account_master_no = $current_supplier_no;
            $account_master_permission->account_master_type = "supplier";
            $account_master_permission->account_master_name = $name;
            $account_master_permission->save();
        }else{
            $account_master_permission = new AccountMasterPermission();
            $account_master_permission->user_id = Auth::user()->id;
            $account_master_permission->branch_id = $selectedBranchId;
            $account_master_permission->account_master_id = $current_supplier_id;
            $account_master_permission->account_master_no = $current_supplier_no;
            $account_master_permission->account_master_type = "supplier";
            $account_master_permission->account_master_name = $name;
            $account_master_permission->save();

            $account_master_permission = new AccountMasterPermission();
            $account_master_permission->user_id = Auth::user()->admin_id;
            $account_master_permission->branch_id = $selectedBranchId;
            $account_master_permission->account_master_id = $current_supplier_id;
            $account_master_permission->account_master_no = $current_supplier_no;
            $account_master_permission->account_master_type = "supplier";
            $account_master_permission->account_master_name = $name;
            $account_master_permission->save();
        }

        flash()->addSuccess('Record Updated');
        return redirect()->route('software.suppliers.index');
    } catch (\Throwable $th) {
        flash()->addError($th->getMessage());
        return redirect()->back()->withInput();
    }
}


    public function softDelete($supplier)
    {
        $supplier = Supplier::findOrFail($supplier);
        $supplier->delete();
        flash()->addSuccess('Success');
        return redirect()->route('software.suppliers.index');
    }

    public function show($supplier)
    {
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();

        $branchId = session('selected_branch');

        $account_master_permission = AccountMasterPermission::where([
            ['branch_id', $branchId],
            ['account_master_id', $supplier],
            ['account_master_type', 'supplier']
        ])->pluck('user_id');

        if(!empty(($branchId = session()->get('selected_branch'))) && !Branches::hasBranchPermission($branchId, 'customers-account-master')) return response()->json(['status'=>false, 'error'=>'Unauthorized'], 403);
        $supplier = Supplier::with('country', 'classOfAccount')->findOrFail($supplier);
        return view('crusher.supplier_master.view', compact('supplier','subadmins','account_master_permission'));
    }
}
